<?php
/* BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */
?>
<?php

/**
 * Archive
 *
 * Plugin that adds a new button to the mailbox toolbar
 * to archive messages.
 *
 * @version @package_version@
 * @license GNU GPLv3+
 * @author Andre Rodier, Thomas Bruederli
 */
class bm_archive extends rcube_plugin
{
  public $client; 

  public $rcmail;

  public function init() {
    $this->rcmail = rcmail::get_instance();
    $this->add_hook('startup', array($this, 'startup'));
  }

  private function register() {
    $rcmail = rcmail::get_instance();

    $this->add_hook('messages_list', array($this, 'message_list'));
    $this->register_action('plugin.bm_archive.demote', array($this, 'demote'));
    $this->register_action('plugin.bm_archive.promote', array($this, 'promote'));
    $this->client = new BM\HSMClient($_SESSION['bm']['core'], 
        $rcmail->decrypt($_SESSION['password']), $_SESSION['bm_sso']['bmDomain']);
    $this->include_script('bm_archive.js');
    $this->rcmail->output->set_env('archive_enabled', true);
    $this->add_texts('localization', true);
  }

  public function startup() {
    if ($this->isEnabled()) {
      $this->register();
      if ($this->rcmail->task == 'mail' && $this->rcmail->action == '') {
        $this->api->output->add_label('bm_archive.demote_title');
        $this->api->output->add_label('bm_archive.promote_title');
        $this->add_button(
          array(
              'type' => 'link',
              'label' => 'demote_text',
              'command' => 'plugin.bm_archive',
              'class' => 'goog-inline-block goog-button goog-button-base goog-button-base-disabled btn fa fa-archive',
              'classact' => 'goog-inline-block goog-button goog-button-base btn fa fa-archive',
              'title' => 'demote_title',
              'content' => '&nbsp;',
              'domain' => $this->ID
          ),
          'toolbar');
      }
    }
  }

  public function message_list($args) {
    $count = count($args['messages']);
    for ($i=0;$i<$count;$i++) {
      $header = $args['messages'][$i];
      $uid = $header->uid;
      if ($uid) {
        $archived = ($header->get('x-bm_hsm_id') != null);
        $header->list_flags['extra_flags']['archive'] = $archived;
	if ($archived) {
	  $header->list_flags['extra_flags']['hsm'] = $header->get('x-bm_hsm_id');
        }
        $span = html::span(array("class" => ($archived ? 'fa fa-archive':''), "title" => ($archived ? $this->gettext('archived'):''))," ");
        
        $args['messages'][$i]->list_cols['archive'] = $span;
        
      }
    }
    return $args;
  }

  public function demote() {
    $this->add_texts('localization');

    $uid = get_input_value('_uid', RCUBE_INPUT_POST);
    $imapid = get_input_value('_imapid', RCUBE_INPUT_POST);
    $mbox = get_input_value('_mbox', RCUBE_INPUT_POST);
    $mbox = rcube_charset_convert($mbox, 'UTF7-IMAP', RCMAIL_CHARSET);    
    $imapid = $this->rcmail->get_storage()->getImapUID($imapid);

    $demote = new BM\Demote();
    $demote->mailboxUid = $_SESSION['bm_sso']['bmUserId']; 
    $demote->folder = $mbox;
    $demote->imapId = $imapid;

    $tcr = $this->client->demote($demote);
    if ($tcr->errorType != null) {
      if(strpos($tcr->message, 'quota exceeded') !== false ) {
        $this->rcmail->output->command('display_message', $this->gettext('quota_excedeed'), 'error');
      } else {
        $this->rcmail->output->command('display_message', $tcr->message, 'error');
      }
    } else {
      $this->rcmail->output->command('display_message', $this->gettext('demote_msg'), 'confirmation');
      $this->rcmail->output->command('demote_message', $uid, $tcr->imapId, $tcr->hsmId);
    }
    $this->rcmail->output->send();
  }

  public function promote() {
    $this->add_texts('localization');

    $uid = get_input_value('_uid', RCUBE_INPUT_POST);
    $hsm = get_input_value('_hsm', RCUBE_INPUT_POST);
    $mbox = get_input_value('_mbox', RCUBE_INPUT_POST);
    $mbox = rcube_charset_convert($mbox, 'UTF7-IMAP', RCMAIL_CHARSET);    
    $imapid = get_input_value('_imapid', RCUBE_INPUT_POST);
    $imapid = $this->rcmail->get_storage()->getImapUID($imapid);

    $promote = new BM\Promote();
    $promote->mailboxUid = $_SESSION['bm_sso']['bmUserId'];
    $promote->folder = $mbox;
    $promote->hsmId = $hsm;
    $promote->imapUid = $imapid;

    $tcr = $this->client->promote($promote);

    $this->rcmail->output->command('display_message', $this->gettext('promote_msg'), 'confirmation');
    $this->rcmail->output->command('promote_message', $uid, $tcr->imapId);
    $this->rcmail->output->send();
  }

  public function isEnabled() {
    if (!isset($_SESSION['bm']['archive']['enabled'])) {
      $ini_array = parse_ini_file("/etc/bm/bm.ini");
      $locator = new LocatorService($ini_array['locator'] ? $ini_array['locator'] : $ini_array['host']);
      try {
        // FIXME is allowed hsm/demote
        $isAllowed = true;
        $enabled =  $isAllowed && $locator->get('mail/archive', $_SESSION['bm_sso']['bmLogin']);
      } catch (Exception $e) {
        $enabled = false;
      }
      $_SESSION['bm']['archive']['enabled'] = $enabled;
    }
    return $_SESSION['bm']['archive']['enabled'];
  }
}
